@@ -2,7 +2,11 @@
 # License: GPLv3 Copyright: 2024, Kovid Goyal <kovid at kovidgoyal.net>
 
 from qt.core import (
+    QAbstractItemView,
     QCheckBox,
+    QDialog,
+    QDoubleSpinBox,
+    QFont,
     QFormLayout,
     QHBoxLayout,
     QIcon,
@@ -12,6 +16,8 @@ from qt.core import (
     QPushButton,
     QSize,
     QSlider,
+    QStyle,
+    QStyleOptionViewItem,
     Qt,
     QTreeWidget,
     QTreeWidgetItem,
@@ -21,6 +27,7 @@ from qt.core import (
 )
 
 from calibre.gui2.tts.types import (
+    TTS_EMBEDED_CONFIG,
     AudioDeviceId,
     EngineMetadata,
     EngineSpecificSettings,
@@ -70,6 +77,26 @@ class EngineChoice(QWidget):
         self.engine_description.setText(metadata.description)
 
 
+class SentenceDelay(QDoubleSpinBox):
+
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        self.setRange(0., 2.)
+        self.setDecimals(2)
+        self.setSuffix(_(' seconds'))
+        self.setToolTip(_('The number of seconds to pause for at the end of a sentence.'))
+        self.setSpecialValueText(_('no pause'))
+        self.setSingleStep(0.05)
+
+    @property
+    def val(self) -> str:
+        return max(0.0, self.value())
+
+    @val.setter
+    def val(self, v) -> None:
+        self.setValue(float(v))
+
+
 class FloatSlider(QSlider):
 
     def __init__(self, minimum: float = -1, maximum: float = 1, factor: int = 10, parent=None):
@@ -131,27 +158,81 @@ class Voices(QTreeWidget):
 
     voice_changed = pyqtSignal()
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, for_embedding=False):
+        self.for_embedding = for_embedding
         super().__init__(parent)
         self.setHeaderHidden(True)
         self.system_default_voice = Voice()
         self.currentItemChanged.connect(self.voice_changed)
+        self.normal_font = f = self.font()
+        self.highlight_font = f = QFont(f)
+        f.setBold(True), f.setItalic(True)
+        self.ignore_item_changes = False
+        if self.for_embedding:
+            self.setSelectionMode(QAbstractItemView.SelectionMode.NoSelection)
+            self.itemChanged.connect(self.item_changed)
+
+    def item_changed(self, item: QTreeWidgetItem, column: int):
+        if column == 0 and self.is_voice_item(item) and not self.ignore_item_changes:
+            if item.checkState(0) == Qt.CheckState.Checked:
+                p = item.parent()
+                if p is not None:
+                    for child in (p.child(i) for i in range(p.childCount())):
+                        if child is not item and child.checkState(0) == Qt.CheckState.Checked:
+                            self.ignore_item_changes = True
+                            child.setCheckState(0, Qt.CheckState.Unchecked)
+                            self.ignore_item_changes = False
+
+    def is_voice_item(self, item):
+        return item is not None and isinstance(item.data(0, Qt.ItemDataRole.UserRole), Voice)
+
+    def mousePressEvent(self, event):
+        item = self.itemAt(event.pos())
+        if self.for_embedding and self.is_voice_item(item):
+            rect = self.visualItemRect(item)
+            x = event.pos().x() - (rect.x() + self.frameWidth())
+            option = QStyleOptionViewItem()
+            self.initViewItemOption(option)
+            option.rect = rect
+            option.features |= QStyleOptionViewItem.ViewItemFeature.HasCheckIndicator
+            checkbox_rect = self.style().subElementRect(QStyle.SubElement.SE_ItemViewItemCheckIndicator, option, self)
+            if x > checkbox_rect.width():
+                item.setCheckState(0, Qt.CheckState.Checked if item.checkState(0) != Qt.CheckState.Checked else Qt.CheckState.Unchecked)
+        super().mousePressEvent(event)
 
     def sizeHint(self) -> QSize:
         return QSize(400, 500)
 
-    def set_voices(self, all_voices: tuple[Voice, ...], current_voice: str, engine_metadata: EngineMetadata) -> None:
+    def set_item_downloaded_state(self, ans: QTreeWidgetItem) -> None:
+        voice = ans.data(0, Qt.ItemDataRole.UserRole)
+        if isinstance(voice, Voice):
+            is_downloaded = bool(voice and voice.engine_data and voice.engine_data.get('is_downloaded'))
+            ans.setFont(0, self.highlight_font if is_downloaded else self.normal_font)
+
+    def set_voices(
+        self, all_voices: tuple[Voice, ...], current_voice: str, engine_metadata: EngineMetadata,
+        preferred_voices: dict[str, str] | None = None
+    ) -> None:
         self.clear()
+        if self.for_embedding:
+            current_voice = ''
+            preferred_voices = preferred_voices or {}
         current_item = None
         def qv(parent, voice):
             nonlocal current_item
-            ans = QTreeWidgetItem(parent, [voice.short_text(engine_metadata)])
+            text = voice.short_text(engine_metadata)
+            ans = QTreeWidgetItem(parent, [text])
             ans.setData(0, Qt.ItemDataRole.UserRole, voice)
             ans.setToolTip(0, voice.tooltip(engine_metadata))
+            if self.for_embedding:
+                ans.setFlags(Qt.ItemFlag.ItemIsUserCheckable | Qt.ItemFlag.ItemIsEnabled)
+                ans.setCheckState(0, Qt.CheckState.Unchecked)
             if current_voice == voice.name:
                 current_item = ans
+            self.set_item_downloaded_state(ans)
             return ans
-        qv(self.invisibleRootItem(), self.system_default_voice)
+        if not self.for_embedding:
+            qv(self.invisibleRootItem(), self.system_default_voice)
         vmap = {}
         for v in all_voices:
             vmap.setdefault(v.language_code, []).append(v)
@@ -165,9 +246,12 @@ class Voices(QTreeWidget):
             parent = parent_map.get(langcode)
             if parent is None:
                 parent_map[langcode] = parent = QTreeWidgetItem(self.invisibleRootItem(), [lang(langcode)])
-                parent.setFlags(parent.flags() & ~Qt.ItemFlag.ItemIsSelectable)
+                parent.setFlags(parent.flags() & ~Qt.ItemFlag.ItemIsSelectable & ~Qt.ItemFlag.ItemIsUserCheckable)
+                parent.setData(0, Qt.ItemDataRole.UserRole, langcode)
             for voice in vmap[langcode]:
-                qv(parent, voice)
+                v = qv(parent, voice)
+                if self.for_embedding and voice.name and preferred_voices.get(langcode) == voice.name:
+                    v.setCheckState(0, Qt.CheckState.Checked)
         if current_item is not None:
             self.setCurrentItem(current_item)
 
@@ -176,19 +260,40 @@ class Voices(QTreeWidget):
         voice = self.current_voice
         return voice.name if voice else ''
 
+    @property
+    def preferred_voices(self) -> dict[str, str] | None:
+        r = self.invisibleRootItem()
+        ans = {}
+        for parent in (r.child(i) for i in range(r.childCount())):
+            langcode = parent.data(0, Qt.ItemDataRole.UserRole)
+            for child in (parent.child(i) for i in range(parent.childCount())):
+                if child.checkState(0) == Qt.CheckState.Checked:
+                    voice = child.data(0, Qt.ItemDataRole.UserRole)
+                    if voice.name:
+                        ans[langcode] = voice.name
+        return ans or None
+
     @property
     def current_voice(self) -> Voice | None:
         ci = self.currentItem()
         if ci is not None:
-            return ci.data(0, Qt.ItemDataRole.UserRole)
+            ans = ci.data(0, Qt.ItemDataRole.UserRole)
+            if isinstance(ans, Voice):
+                return ans
+
+    def refresh_current_item(self) -> None:
+        ci = self.currentItem()
+        if self.is_voice_item(ci):
+            self.set_item_downloaded_state(ci)
 
 
 class EngineSpecificConfig(QWidget):
 
     voice_changed = pyqtSignal()
 
-    def __init__(self, parent):
+    def __init__(self, parent: QWidget = None, for_embedding: bool = False):
         super().__init__(parent)
+        self.for_embedding = for_embedding
         self.engine_name = ''
         self.l = l = QFormLayout(self)
         devs = QMediaDevices.audioOutputs()
@@ -204,15 +309,17 @@ class EngineSpecificConfig(QWidget):
         self.engine_specific_settings = {}
         self.rate = r = FloatSlider(parent=self)
         l.addRow(_('&Speed of speech:'), r)
+        self.sentence_delay = d = SentenceDelay(parent=self)
+        l.addRow(_('&Pause after sentence:'), d)
         self.pitch = p = FloatSlider(parent=self)
         l.addRow(_('&Pitch of speech:'), p)
         self.volume = v = Volume(self)
         l.addRow(v)
         self.audio_device = ad = QComboBox(self)
         l.addRow(_('Output a&udio to:'), ad)
-        self.voices = v = Voices(self)
+        self.voices = v = Voices(self, self.for_embedding)
         v.voice_changed.connect(self.voice_changed)
-        la = QLabel(_('V&oices:'))
+        la = QLabel(_('Choose &default voice for language:') if self.for_embedding else _('V&oices:'))
         la.setBuddy(v)
         l.addRow(la)
         l.addRow(v)
@@ -226,7 +333,10 @@ class EngineSpecificConfig(QWidget):
         tts = create_tts_backend(force_engine=engine_name)
         if engine_name not in self.voice_data:
             self.voice_data[engine_name] = tts.available_voices
-            self.engine_specific_settings[engine_name] = EngineSpecificSettings.create_from_config(engine_name)
+            if self.for_embedding:
+                self.engine_specific_settings[engine_name] = EngineSpecificSettings.create_from_config(engine_name, TTS_EMBEDED_CONFIG)
+            else:
+                self.engine_specific_settings[engine_name] = EngineSpecificSettings.create_from_config(engine_name)
             self.default_output_modules[engine_name] = tts.default_output_module
         self.output_module.blockSignals(True)
         self.output_module.clear()
@@ -240,6 +350,7 @@ class EngineSpecificConfig(QWidget):
         else:
             self.layout().setRowVisible(self.output_module, False)
         self.output_module.blockSignals(False)
+        self.layout().setRowVisible(self.sentence_delay, metadata.has_sentence_delay)
         try:
             s = self.engine_specific_settings[self.engine_name]
         except KeyError:
@@ -252,14 +363,16 @@ class EngineSpecificConfig(QWidget):
             self.pitch.val = 0
             self.layout().setRowVisible(self.pitch, False)
         self.layout().setRowVisible(self.pitch, metadata.can_change_pitch)
-        if metadata.can_change_volume:
+        if metadata.can_change_volume and not self.for_embedding:
             self.layout().setRowVisible(self.volume, True)
             self.volume.val = s.volume
         else:
             self.layout().setRowVisible(self.volume, False)
             self.volume.val = None
+        if metadata.has_sentence_delay:
+            self.sentence_delay.val = s.sentence_delay
         self.audio_device.clear()
-        if metadata.allows_choosing_audio_device:
+        if metadata.allows_choosing_audio_device and not self.for_embedding:
             self.audio_device.addItem(_('System default (currently {})').format(self.default_audio_device.description), '')
             for ad in self.all_audio_devices:
                 self.audio_device.addItem(ad.description, ad.id.hex())
@@ -282,13 +395,19 @@ class EngineSpecificConfig(QWidget):
         if metadata.has_multiple_output_modules:
             output_module = output_module or self.default_output_modules[self.engine_name]
         all_voices = self.voice_data[self.engine_name][output_module]
-        self.voices.set_voices(all_voices, s.voice_name, metadata)
+        self.voices.set_voices(all_voices, s.voice_name, metadata, s.preferred_voices)
 
     def as_settings(self) -> EngineSpecificSettings:
         ans = EngineSpecificSettings(
             engine_name=self.engine_name,
-            rate=self.rate.val, voice_name=self.voices.val, pitch=self.pitch.val, volume=self.volume.val)
+            rate=self.rate.val, pitch=self.pitch.val, volume=self.volume.val)
+        if self.for_embedding:
+            ans = ans._replace(preferred_voices=self.voices.preferred_voices)
+        else:
+            ans = ans._replace(voice_name=self.voices.val)
         metadata = available_engines()[self.engine_name]
+        if metadata.has_sentence_delay:
+            ans = ans._replace(sentence_delay=self.sentence_delay.val)
         if metadata.has_multiple_output_modules and self.output_module.currentIndex() > 0:
             ans = ans._replace(output_module=self.output_module.currentData())
         if metadata.allows_choosing_audio_device and self.audio_device.currentIndex() > 0:
@@ -357,6 +476,7 @@ class ConfigDialog(Dialog):
         else:
             b.setIcon(QIcon.ic('download-metadata.png'))
             b.setText(_('Download voice'))
+        self.engine_specific_config.voices.refresh_current_item()
 
     def voice_action(self):
         self.engine_specific_config.voice_action()
@@ -382,6 +502,42 @@ class ConfigDialog(Dialog):
         super().accept()
 
 
+class EmbeddingConfig(QWidget):
+
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        self.l = l = QVBoxLayout(self)
+        self.engine_specific_config = esc = EngineSpecificConfig(self, for_embedding=True)
+        l.addWidget(esc)
+        self.engine_specific_config.set_engine('piper')
+
+    def save_settings(self):
+        s = self.engine_specific_config.as_settings()
+        prefs = load_config(TTS_EMBEDED_CONFIG)
+        with prefs:
+            s.save_to_config(prefs, TTS_EMBEDED_CONFIG)
+
+
+def develop_embedding():
+    class D(Dialog):
+        def __init__(self, parent=None):
+            super().__init__('Configure Text to speech audio generation', 'configure-tts-embed', parent=parent)
+
+        def setup_ui(self):
+            self.l = l = QVBoxLayout(self)
+            self.conf = c = EmbeddingConfig(self)
+            l.addWidget(c)
+            l.addWidget(self.bb)
+
+    from calibre.gui2 import Application
+    app = Application([])
+    d = D()
+    if d.exec() == QDialog.DialogCode.Accepted:
+        d.conf.save_settings()
+    del d
+    del app
+
+
 def develop():
     from calibre.gui2 import Application
     app = Application([])
@@ -392,4 +548,4 @@ def develop():
 
 
 if __name__ == '__main__':
-    develop()
+    develop_embedding()
